#!/usr/bin/env python3
"""
Created on Mon Jun 12 22:21:47 2023.

@author: ghiggi
"""


def get_dataset_partitioning_columns(fpath):
    """Infer partitioning columns from dataset file path."""
    # TODO: implement logic for that
    partitioning_columns = ["year", "month"]
    return partitioning_columns
