#!/usr/bin/env python3
"""
Created on Mon Jun 12 22:27:07 2023.

@author: ghiggi
"""


def get_time_filters(start_time=None, end_time=None):
    """Define filters for Parquet Dataset subsetting at read-time."""
    # TODO implement logic
    filters = None
    return filters
